#include "PCH.h"

#include "AudioMgrImpl.h"
#include "SoundMgrImpl.h"
#include "MusicMgrImpl.h"
#include <stdexcept>

//========================================================================
UQ::AudioMgrImpl::AudioMgrImpl() :
	volume_(0.8f),
	pSoundMgr_(nullptr),
	pMusicMgr_(nullptr)
{
	if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 1024) < 0){
		throw std::runtime_error("SDL_mixer ̏Ɏs܂B");
	}

	pSoundMgr_ = std::unique_ptr<SoundMgrImpl>(new SoundMgrImpl(this));
	pMusicMgr_ = std::unique_ptr<MusicMgrImpl>(new MusicMgrImpl(this));

	volume_ = std::atoi(iniFile.GetKeyValue("Audio", "volume", makeString("%d", int(volume_ * 100))).c_str()) / 100.f;
	if(volume_ > 1.f) volume_ = 1.f;
	if(volume_ < 0.f) volume_ = 0.f;
}

//========================================================================
UQ::AudioMgrImpl::~AudioMgrImpl()
{
	iniFile.SetKeyValue("Audio", "volume", makeString("%d", int(volume_ * 100)));
	Mix_CloseAudio();
	SDL_QuitSubSystem(SDL_INIT_AUDIO);
}

//========================================================================
void UQ::AudioMgrImpl::setVolume(float volume)
{
	volume_ = volume;
	pMusicMgr_->resetVolume();
}

//========================================================================
UQ::SoundMgr* UQ::AudioMgrImpl::getSoundMgr(void) const{return pSoundMgr_.get();}
UQ::MusicMgr* UQ::AudioMgrImpl::getMusicMgr(void) const{return pMusicMgr_.get();}

