#include "PCH.h"

#include "GraphicMgrImpl.h"
#include <stdexcept>
#include <cmath>

namespace{
static const int TEX_FONT_ID = 0x8000;
}
//========================================================================
UQ::GraphicMgrImpl::GraphicMgrImpl(int windowWidth, int windowHeight, const char* systemFontFileName, int systemFontWidth, int systemFontHeight) :
	pCurrentTexture_(nullptr),
	textures_(),
	isFullScreen_(true),
	windowWidth_(windowWidth), windowHeight_(windowHeight),
	fullScreenWidth_(0), fullScreenHeight_(0),
	xShift_(0), yShift_(0),
	systemFontWidth_(systemFontWidth), systemFontHeight_(systemFontHeight)
{
	isFullScreen_ = (std::atoi(iniFile.GetKeyValue("System", "isFullScreen", (isFullScreen_) ? "1" : "0").c_str()) == 1);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
	SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
	setVideoMode();
	loadTexture(TEX_FONT_ID, systemFontFileName);
}

//========================================================================
UQ::GraphicMgrImpl::~GraphicMgrImpl()
{
	iniFile.SetKeyValue("System", "isFullScreen", (isFullScreen_) ? "1" : "0");
	SDL_QuitSubSystem(SDL_INIT_VIDEO);
}

//========================================================================
void UQ::GraphicMgrImpl::setVideoMode(void)
{
	if(isFullScreen_){
		static int screenSize[][2] = {
			{640, 480}, {800, 600}, {1024, 768}, {1152, 864}, {1280, 960}, {1280, 1024}, {1600, 1200}
		};
		for(int i = 0; i < sizeof(screenSize) / sizeof(int) / 2; i++){
			int w = screenSize[i][0], h = screenSize[i][1];
			if(windowWidth_ <= w && windowHeight_ <= h){
				if(SDL_VideoModeOK(w, h, 16, SDL_OPENGL | SDL_FULLSCREEN)){
					fullScreenWidth_ = w; fullScreenHeight_ = h;
					xShift_ = (fullScreenWidth_ - windowWidth_) / 2;
					yShift_ = (fullScreenHeight_ - windowHeight_) / 2;
					break;
				}
			}
		}
	}else{
		fullScreenWidth_ = windowWidth_;
		fullScreenHeight_ = windowHeight_;
		xShift_ = 0; yShift_ = 0;
	}
	if(!SDL_SetVideoMode(fullScreenWidth_, fullScreenHeight_, 16, SDL_OPENGL | ((isFullScreen_) ? SDL_FULLSCREEN : 0))){
		throw std::runtime_error("ʂ̏Ɏs܂B");
	}
}

//========================================================================
void UQ::GraphicMgrImpl::changeDisplayMode(void)
{
	isFullScreen_ = !isFullScreen_;
	setVideoMode();
	reloadAllTexture();
}

//========================================================================
void UQ::GraphicMgrImpl::drawPolygon(Polygon polygon) const
{
	if(polygon.vertexCount_ < 3) return;
	polygon.translate(Vector2D((float)xShift_, (float)yShift_));
	if(pCurrentTexture_){
		int twid = pCurrentTexture_->getWidth(), thei = pCurrentTexture_->getHeight();
		glEnable(GL_TEXTURE_2D); 
		glBindTexture(GL_TEXTURE_2D, pCurrentTexture_->getTexture());
		glBegin(GL_TRIANGLE_STRIP);
			for(int i = 0; i < polygon.vertexCount_; i++){
				glColor4ub(polygon.vertex_[i].color_.r_, polygon.vertex_[i].color_.g_, polygon.vertex_[i].color_.b_, polygon.vertex_[i].color_.a_);
				glTexCoord2f((float)polygon.vertex_[i].textureCoordinate_.x_ / twid, (float)polygon.vertex_[i].textureCoordinate_.y_ / thei);
				glVertex2f(polygon.vertex_[i].vector_.x_, polygon.vertex_[i].vector_.y_);
			}
		glEnd();
		glBindTexture(GL_TEXTURE_2D, 0);
		glDisable(GL_TEXTURE_2D); 
	}else{
		glBegin(GL_TRIANGLE_STRIP);
			for(int i = 0; i < polygon.vertexCount_; i++){
				glColor4ub(polygon.vertex_[i].color_.r_, polygon.vertex_[i].color_.g_, polygon.vertex_[i].color_.b_, polygon.vertex_[i].color_.a_);
				glVertex2f(polygon.vertex_[i].vector_.x_, polygon.vertex_[i].vector_.y_);
			}
		glEnd();
	}
}

//========================================================================
bool UQ::GraphicMgrImpl::loadTexture(int id, const char* fileName)
{
	TexturePtr pTexture = TexturePtr(new Texture());
	if(!pTexture->load(fileName)){
		pTexture = nullptr;
	}
	MY_VERIFY_EXPR(textures_.insert(Textures::value_type(id, pTexture)).second, "w肳ꂽID͓o^ς݂łB");
	return (pTexture != nullptr);
}
//========================================================================
void UQ::GraphicMgrImpl::reloadAllTexture(void)
{
	for(Textures::iterator i = textures_.begin(); i != textures_.end(); ++i){
		if(i->second) i->second->reload();
	}
}

//========================================================================
void UQ::GraphicMgrImpl::unloadTexture(int id)
{
	Textures::iterator i = textures_.find(id);
	MY_ASSERT_EXPR(i != textures_.end(), "w肳ꂽID͓o^Ă܂B");
	textures_.erase(id);
}

//========================================================================
void UQ::GraphicMgrImpl::setTexture(int id)
{
	Textures::iterator i = textures_.find(id);
	MY_ASSERT_EXPR(i != textures_.end(), "w肳ꂽID͓o^Ă܂B");
	pCurrentTexture_ = i->second.get();
}

//========================================================================
void UQ::GraphicMgrImpl::unsetTexture(void)
{
	pCurrentTexture_ = nullptr;
}

//========================================================================
int UQ::GraphicMgrImpl::getTextureWidth(int id) const
{
	Textures::const_iterator i = textures_.find(id);
	MY_ASSERT_EXPR(i != textures_.end(), "w肳ꂽID͓o^Ă܂B");
	if(i->second) return i->second->getWidth();
	return 256;
}

//========================================================================
int UQ::GraphicMgrImpl::getTextureHeight(int id) const
{
	Textures::const_iterator i = textures_.find(id);
	MY_ASSERT_EXPR(i != textures_.end(), "w肳ꂽID͓o^Ă܂B");
	if(i->second) return i->second->getHeight();
	return 256;
}

//========================================================================
void UQ::GraphicMgrImpl::drawWindowBox(void)
{
	if(xShift_ == 0 && yShift_ == 0) return;
	
	unsetTexture();
	Polygon rect;
	rect.setColor(0xFF000000);

	rect.setRectangle(-xShift_, -yShift_, xShift_, fullScreenHeight_);
	drawPolygon(rect);
	rect.setRectangle(fullScreenWidth_ - 2 * xShift_, -yShift_, xShift_, fullScreenHeight_);
	drawPolygon(rect);
	rect.setRectangle(0, -yShift_, fullScreenWidth_ - 2 * xShift_, yShift_);
	drawPolygon(rect);
	rect.setRectangle(0, fullScreenHeight_ - 2 * yShift_, fullScreenWidth_ - 2 * xShift_, yShift_);
	drawPolygon(rect);
}

//========================================================================
void UQ::GraphicMgrImpl::SDL_GL_Enter2DMode(void)
{
	SDL_Surface *screen = SDL_GetVideoSurface();
 
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);

	glViewport(0, 0, screen->w, screen->h);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(0.0, (GLdouble)screen->w, (GLdouble)screen->h, 0.0, 0.0, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}
void UQ::GraphicMgrImpl::SDL_GL_Leave2DMode(void)
{
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glPopAttrib();
}


//========================================================================
void UQ::GraphicMgrImpl::outputString(int x, int y, const char* str, Color color)
{
	setTexture(TEX_FONT_ID);
	Polygon rect;
	for(unsigned int i = 0; i < strlen(str); i++){
		rect.setRectangle(x + i * systemFontWidth_, y, systemFontWidth_, systemFontHeight_);
		rect.setRectangleTexture((str[i] % 16) * systemFontWidth_, (str[i] / 16) * systemFontHeight_, systemFontWidth_, systemFontHeight_);
		rect.setColor(color);
		drawPolygon(rect);
	}
}

//========================================================================
//========================================================================
UQ::Texture::Texture() :
	texture_(0),
	width_(0),
	height_(0)
{
}

//========================================================================
UQ::Texture::~Texture()
{
	glDeleteTextures(1, &texture_);
}

//========================================================================
bool UQ::Texture::load(const char* fileName)
{
	strcpy(fileName_, fileName);

	pngInfo info;
	texture_ = pngBind(fileName, PNG_NOMIPMAP, PNG_ALPHA, &info, GL_CLAMP, GL_NEAREST, GL_NEAREST);
	if(texture_ == 0) return false;
	width_ = info.Width;
	height_ = info.Height;
	return true;
}

//========================================================================
//========================================================================
void UQ::Polygon::setRectangle(float x, float y, int width, int height)
{
	vertexCount_ = 4;
	vertex_[0].vector_.set(x        , y         );
	vertex_[1].vector_.set(x + width, y         );
	vertex_[2].vector_.set(x        , y + height);
	vertex_[3].vector_.set(x + width, y + height);
}
void UQ::Polygon::setRectangle(int x, int y, int width, int height)
{
	setRectangle((float)x, (float)y, width, height);
}

//========================================================================
void UQ::Polygon::setRectangleTexture(int x, int y, int width, int height)
{
	vertex_[0].textureCoordinate_.set(x        , y         );
	vertex_[1].textureCoordinate_.set(x + width, y         );
	vertex_[2].textureCoordinate_.set(x        , y + height);
	vertex_[3].textureCoordinate_.set(x + width, y + height);
}

//========================================================================
void UQ::Polygon::setRegularPolygon(int n, UQ::Vector2D center, float radius)
{
	MY_ASSERT_EXPR(n <= VERTEX_COUNT_MAX_, "_̐ UQ::Polygon::VERTEX_COUNT_MAX_ ȏɐݒ肳܂B");
	vertexCount_ = n;
	vertex_[0].vector_ = center + Vector2D(radius, 0);
	for(int i = 1; i < (n + 1) / 2; i++){
		vertex_[i * 2 - 1].vector_ = center + Vector2D(cos(PI2 / n * i      ), sin(PI2 / n * i      )) * radius;
		vertex_[i * 2    ].vector_ = center + Vector2D(cos(PI2 / n * (n - i)), sin(PI2 / n * (n - i))) * radius;
	}
	if(n % 2 == 0) vertex_[n - 1].vector_ = center + Vector2D(-radius, 0);
}

//========================================================================
void UQ::Polygon::setColor(UQ::Color color)
{
	for(int i = 0; i < VERTEX_COUNT_MAX_; i++){
		vertex_[i].color_ = color;
	}
}

//========================================================================
void UQ::Polygon::translate(Vector2D vector)
{
	for(int i = 0; i < vertexCount_; i++){
		vertex_[i].vector_ += vector;
	}
}

//========================================================================
void UQ::Polygon::rotate(Vector2D center, float angle)
{
	for(int i = 0; i < vertexCount_; i++){
		Vector2D v = vertex_[i].vector_ - center;
		v.set(v.x_ * cos(angle) - v.y_ * sin(angle), v.x_ * sin(angle) + v.y_ * cos(angle));
		vertex_[i].vector_ = center + v;
	}
}

//========================================================================
void UQ::Polygon::scale(Vector2D center, float scaleX, float scaleY)
{
	for(int i = 0; i < vertexCount_; i++){
		Vector2D v = vertex_[i].vector_ - center;
		v.x_ *= scaleX;
		v.y_ *= scaleY;
		vertex_[i].vector_ = center + v;
	}
}
