#pragma once

#include "common.h"
#include "../Include/GraphicMgr.h"

class CIniFileA;

namespace UQ{

class Texture{
public:
	Texture();
	~Texture();
	bool load(const char* fileName);
	bool reload(void){return load(fileName_);}
	GLuint getTexture(void) const{return texture_;}
	int getWidth(void) const{return width_;}
	int getHeight(void) const{return height_;}
private:
	char fileName_[100];
	GLuint texture_;
	int width_, height_;
};

class GraphicMgrImpl : public GraphicMgr{
private:
	typedef std::shared_ptr<Texture> TexturePtr;
	typedef std::map<int, TexturePtr> Textures;

public:
	GraphicMgrImpl(int windowWidth, int windowHeight, const char* systemFontFileName, int systemFontWidth, int systemFontHeight);
	~GraphicMgrImpl();

	void setVideoMode(void);
	void changeDisplayMode(void);

	void setBackgroundColor(Color color){glClearColor((GLclampf)color.r_ / 0xFF, (GLclampf)color.g_ / 0xFF, (GLclampf)color.b_ / 0xFF, 0.f);}
	void begin(void){glClear(GL_COLOR_BUFFER_BIT); SDL_GL_Enter2DMode();}
	void end(void){SDL_GL_Leave2DMode(); SDL_GL_SwapBuffers();}

	void drawPolygon(Polygon polygon) const;

	bool loadTexture(int id, const char* fileName);
	void reloadAllTexture(void);
	void unloadTexture(int id);
	void setTexture(int id);
	void unsetTexture(void);
	int getTextureWidth(int id) const;
	int getTextureHeight(int id) const;

	void drawWindowBox(void);

	void outputString(int x, int y, const char* str, Color color = 0xFFFFFFFF);
	void loadFontTexture(void);
	int getSystemFontWidth(void){return systemFontWidth_;}
	int getSystemFontHeight(void){return systemFontHeight_;}

private:
	void SDL_GL_Enter2DMode(void);
	void SDL_GL_Leave2DMode(void);

private:
	Texture* pCurrentTexture_;
	Textures textures_;

	bool isFullScreen_;
	int windowWidth_, windowHeight_;
	int fullScreenWidth_, fullScreenHeight_;
	int xShift_, yShift_;
	int systemFontWidth_, systemFontHeight_;
};

} // namespace UQ