#include "PCH.h"

#include "MusicMgrImpl.h"
#include "AudioMgrImpl.h"

//========================================================================
UQ::MusicMgrImpl::MusicMgrImpl(AudioMgrImpl* pAudioMgr) :
	pAudioMgr_(pAudioMgr),
	musics_(),
	pCurrentMusic_(nullptr),
	volume_(1.0f)
{
}

//========================================================================
UQ::MusicMgrImpl::~MusicMgrImpl()
{
}

//========================================================================
bool UQ::MusicMgrImpl::open(int id, const char* fileName)
{
	MusicPtr pMusic = MusicPtr(new Music());
	if(!pMusic->open(fileName)){
		pMusic = nullptr;
	}
	MY_VERIFY_EXPR(musics_.insert(Musics::value_type(id, pMusic)).second, "w肳ꂽID͓o^ς݂łB");
	return (pMusic != nullptr);
}

//========================================================================
void UQ::MusicMgrImpl::close(int id)
{
	Musics::iterator i = musics_.find(id);
	MY_ASSERT_EXPR(i != musics_.end(), "w肳ꂽID͓o^Ă܂B");
	musics_.erase(i);
}

//========================================================================
void UQ::MusicMgrImpl::play(int id, bool isLooping)
{
	if(pCurrentMusic_) pCurrentMusic_->stop();
	Musics::iterator i = musics_.find(id);
	MY_ASSERT_EXPR(i != musics_.end(), "w肳ꂽID͓o^Ă܂B");
	pCurrentMusic_ = i->second.get();
	if(!pCurrentMusic_) return;
	resetVolume();
	pCurrentMusic_->play(isLooping);
}

//========================================================================
void UQ::MusicMgrImpl::stop(void)
{
	if(!pCurrentMusic_) return;
	pCurrentMusic_->stop();
	pCurrentMusic_ = nullptr;
}

//========================================================================
void UQ::MusicMgrImpl::pause(void)
{
	if(!pCurrentMusic_) return;
	pCurrentMusic_->pause();
}

//========================================================================
void UQ::MusicMgrImpl::resume(void)
{
	if(!pCurrentMusic_) return;
	pCurrentMusic_->resume();
}

//========================================================================
bool UQ::MusicMgrImpl::isPlaying(void) const
{
	if(!pCurrentMusic_) return false;
	return pCurrentMusic_->isPlaying();
}

//========================================================================
void UQ::MusicMgrImpl::setVolume(float volume)
{
	volume_ = volume;
	Mix_VolumeMusic(int(volume_ * pAudioMgr_->getVolume() * 128));
}

//========================================================================
//========================================================================
UQ::Music::Music() :
	music_(0),
	status_(STOP)
{
}

//========================================================================
UQ::Music::~Music()
{
	Mix_FreeMusic(music_);
}

//========================================================================
bool UQ::Music::open(const char* fileName)
{
	music_ = Mix_LoadMUS(fileName);
	if(!music_) return false;
	return true;
}

//========================================================================
void UQ::Music::play(bool isLooping)
{
	if(status_ == PLAYING) return;
	status_ = PLAYING;
	Mix_PlayMusic(music_, (isLooping) ? -1 : 1);
}

//========================================================================
void UQ::Music::stop(void)
{
	if(status_ == STOP) return;
	status_ = STOP;
	Mix_HaltMusic();
}

//========================================================================
void UQ::Music::pause(void)
{
	if(status_ != PLAYING) return;
	status_ = PAUSE;
	Mix_PauseMusic();
}

//========================================================================
void UQ::Music::resume(void)
{
	if(status_ != PAUSE) return;
	status_ = PLAYING;
	Mix_ResumeMusic();
}

//========================================================================
bool UQ::Music::isPlaying(void) const
{
	return (status_ == PLAYING);
}
