#include "PCH.h"

#include "TaskMgrImpl.h"

struct UQ::Task::Impl{
	typedef std::set<Task*> Children;
	Impl(int priority[]) :
		transition_(CREATED),
		pParent_(0),
		children_()
	{
		for(int i = 0; i < EXECUTION_COUNT; i++){
			priority_[i] = priority[i];
			isPriorityChanged_[i] = false;
			isPaused_[i] = false;
		}
	}
	enum{CREATED, ACTIVE, DELETED} transition_;
	int priority_[EXECUTION_COUNT];
	bool isPriorityChanged_[EXECUTION_COUNT];
	bool isPaused_[EXECUTION_COUNT];
	Task* pParent_;
	Children children_;
	void erase(void);
	void pause_resume(Execution execution, bool isPuasing);
};

//========================================================================
void UQ::Task::Impl::erase(void)
{
	transition_ = DELETED;
	for(Children::iterator i = children_.begin(); i != children_.end(); ++i){
		(*i)->pImpl_->erase();
	}
}
void UQ::Task::Impl::pause_resume(Execution execution, bool isPausing)
{
	isPaused_[execution] = isPausing;
	for(Children::iterator i = children_.begin(); i != children_.end(); ++i){
		(*i)->pImpl_->pause_resume(execution, isPausing);
	}
}
//========================================================================
UQ::Task::Task(int updatePriority, int renderPriority) : pImpl_(nullptr)
{
	int priority[2] = {updatePriority, renderPriority};
	pImpl_ = new Impl(priority);
}
UQ::Task::~Task(){delete pImpl_;}
int UQ::Task::getUpdatePriority(void) const{return pImpl_->priority_[UPDATE];}
int UQ::Task::getRenderPriority(void) const{return pImpl_->priority_[RENDER];}
void UQ::Task::setUpdatePriority(int priority){pImpl_->priority_[UPDATE] = priority; pImpl_->isPriorityChanged_[UPDATE] = true;}
void UQ::Task::setRenderPriority(int priority){pImpl_->priority_[RENDER] = priority; pImpl_->isPriorityChanged_[RENDER] = true;}
void UQ::Task::erase(void){pImpl_->erase(); if(pImpl_->pParent_) pImpl_->pParent_->pImpl_->children_.erase(this);} // ̂߂setB
void UQ::Task::pauseUpdate(void) {pImpl_->pause_resume(UPDATE, true );}
void UQ::Task::resumeUpdate(void){pImpl_->pause_resume(UPDATE, false);}
void UQ::Task::pauseRender(void) {pImpl_->pause_resume(RENDER, true );}
void UQ::Task::resumeRender(void){pImpl_->pause_resume(RENDER, false);}

//========================================================================
//========================================================================
UQ::TaskMgrImpl::TaskMgrImpl() : 
	allTasks_(),
	enumerationTasks_(),
	executionTasks_()
{
}

//========================================================================
UQ::TaskMgrImpl::~TaskMgrImpl()
{
	clear();
	optimize();
}

//========================================================================
void UQ::TaskMgrImpl::clear(void)
{
	for(AllTasks::iterator i = allTasks_.begin(); i != allTasks_.end(); ++i){
		(*i)->pImpl_->transition_ = Task::Impl::DELETED;
	}
}

//========================================================================
void UQ::TaskMgrImpl::optimize(void)
{
	for(int e = 0; e < EXECUTION_COUNT; e++){
		for(ExecutionTasks::iterator i = executionTasks_[e].begin(); i != executionTasks_[e].end(); ){
			if(i->second->pImpl_->transition_ == Task::Impl::DELETED || i->second->pImpl_->isPriorityChanged_[e]){
				executionTasks_[e].erase(i++);
			}else{
				++i;
			}
		}
	}
	for(unsigned int type = 0; type < enumerationTasks_.size(); type++){
		for(EnumerationTasks::iterator i = enumerationTasks_[type].begin(); i != enumerationTasks_[type].end(); ){
			if((*i)->pImpl_->transition_ == Task::Impl::DELETED){
				i = enumerationTasks_[type].erase(i);
			}else{
				++i;
			}
		}
	}

	for(AllTasks::iterator i = allTasks_.begin(); i != allTasks_.end(); ){
		if((*i)->pImpl_->transition_ == Task::Impl::DELETED){
			i = allTasks_.erase(i);
		}else{
			for(int e = 0; e < EXECUTION_COUNT; e++){
				if((*i)->pImpl_->isPriorityChanged_[e]){
					(*i)->pImpl_->isPriorityChanged_[e] = false;
					executionTasks_[e].insert(ExecutionTasks::value_type((*i)->pImpl_->priority_[e], *i));
				}
			}
			if((*i)->pImpl_->transition_ == Task::Impl::CREATED) (*i)->pImpl_->transition_ = Task::Impl::ACTIVE;
			++i;
		}
	}
}

//========================================================================
void UQ::TaskMgrImpl::executeAll(void)
{
	for(int e = 0; e < EXECUTION_COUNT; e++){
		for(ExecutionTasks::iterator i = executionTasks_[e].begin(); i != executionTasks_[e].end(); ++i){
			if(i->second->pImpl_->transition_ == Task::Impl::ACTIVE && !(i->second->pImpl_->isPaused_[e])){
				switch(e){
					case UPDATE: i->second->update(); break;
					case RENDER: i->second->render(); break;
					default: MY_ASSERT_EXPR(0, "Ȏs֐͂ȂI"); break;
				}
			}
		}
	}
}

//========================================================================
void UQ::TaskMgrImpl::insertImpl(unsigned int type, Task* pTask, Task* pParent)
{
	TaskPtr p = TaskPtr(pTask);
	allTasks_.push_back(p);

	if(enumerationTasks_.size() <= type) enumerationTasks_.resize(type + 1);
	enumerationTasks_[type].push_back(p);

	for(int e = 0; e < EXECUTION_COUNT; e++){
		if(pTask->pImpl_->priority_[e] != Task::NOT_INSERT_) executionTasks_[e].insert(ExecutionTasks::value_type(pTask->pImpl_->priority_[e], p));
	}

	pTask->pImpl_->pParent_ = pParent;
	if(pParent) pParent->pImpl_->children_.insert(pTask);
}

//========================================================================
unsigned int UQ::TaskMgrImpl::countImpl(unsigned int type) const
{
	if(enumerationTasks_.size() <= type) return 0;
	return enumerationTasks_[type].size();
}

//========================================================================
UQ::Task* UQ::TaskMgrImpl::enumerateImpl(unsigned int type, unsigned int index) const
{
	if(enumerationTasks_.size() <= type) return nullptr;
	if(enumerationTasks_[type].size() <= index) return nullptr;
	return enumerationTasks_[type][index].get();
}
